/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.web.charts;

import java.awt.Color;
import java.awt.image.BufferedImage;

import java.util.ArrayList;
import java.util.List;
import java.text.DecimalFormat;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.category.DefaultCategoryDataset;

public class ComplexityDistributionChart extends BaseChart {
  private CategoryPlot plot;
  protected DefaultCategoryDataset dataset;
  private List<Color> colors;
  
  private final String AXIS_LABEL;

  public ComplexityDistributionChart(int width, int height, List<String> colorsInHexa) {
    this(width, height, colorsInHexa, null);
  }
  public ComplexityDistributionChart(int width, int height, List<String> colorsInHexa, String valueLabelSuffix) {
    super(width, height);
    initColors(colorsInHexa);

    AXIS_LABEL = "Simple" + makeSpace(width/20) + "Average" + makeSpace(width/20) + "Complex";
    
    dataset = new DefaultCategoryDataset();
    plot = new CategoryPlot();

    configureDomainAxis();
    configureRangeAxis(valueLabelSuffix);
    configureRenderer();
        
    plot.setDataset(dataset);
  }
  
  private void initColors(List<String> colorsInHexa) {
    colors = new ArrayList<Color>(colorsInHexa.size());
    for (String color : colorsInHexa) {
      int rgb = Integer.valueOf(color, 16);
      colors.add(new Color(rgb));
    }
  }

  private void configureRenderer() {
    BarRenderer renderer = new BarRenderer();
    renderer.setDrawBarOutline( true );
    renderer.setSeriesItemLabelsVisible( 0, true );
    renderer.setItemMargin(0);
    plot.setRenderer(renderer);
  }

  private void configureDomainAxis() {   
    CategoryAxis categoryAxis = new CategoryAxis();
    categoryAxis.setLabel(AXIS_LABEL);
    categoryAxis.setLabelFont(getFont());
    categoryAxis.setLabelPaint(BASE_COLOR);     
    categoryAxis.setTickMarksVisible(true);
    categoryAxis.setTickLabelFont(getFont());
    categoryAxis.setTickLabelPaint(BASE_COLOR);    
    plot.setDomainAxis(categoryAxis);
    plot.setDomainGridlinesVisible(false);
  }

  private void configureRangeAxis(String valueLabelSuffix) {
    NumberAxis numberAxis = new NumberAxis();
    numberAxis.setUpperMargin(0.3);
    numberAxis.setTickLabelFont(getFont());
    numberAxis.setTickLabelPaint(BASE_COLOR);
    String suffix = "";
    if (valueLabelSuffix!=null && !"".equals(valueLabelSuffix)) {
      suffix = new StringBuilder().append("'").append(valueLabelSuffix).append("'").toString();
    }
    numberAxis.setNumberFormatOverride(new DecimalFormat("0" + suffix));
    numberAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
    plot.setRangeAxis(numberAxis);
  }
  
  public void addMeasure(int index, String label, Double value ){
    dataset.addValue(value, Integer.valueOf(index), label);
  }  
  
  public void addMeasure(String label, Double value) {
    addMeasure(0, label, value);
  }  
  
  private void setRendererColors() {
    dataset.getColumnCount();
    BarRenderer renderer = (BarRenderer) plot.getRenderer();
    for (int i = 0; i < dataset.getColumnCount(); i++) {
      renderer.setSeriesPaint(i, colors.get(i % colors.size()));
      renderer.setSeriesOutlinePaint(i, SERIE_BORDER_COLOR);
    }
  } 

  protected BufferedImage getChartImage() {    
    setRendererColors();    
    plot.setOutlinePaint(BASE_COLOR);
    plot.setDomainGridlinePaint(BASE_COLOR_LIGHT);
    plot.setRangeGridlinePaint(BASE_COLOR_LIGHT);
    JFreeChart chart = new JFreeChart(null, TextTitle.DEFAULT_FONT, plot, true);
    configureChart(chart, false);
    return super.getBufferedImage(chart);
  }
  
  private String makeSpace(int nbSpace){
    String space = "";
    for (int i=0; i<nbSpace; i++) {
      space += " ";
    }    
    return space;
  }  
}