/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.web;

import org.picocontainer.Characteristics;
import org.picocontainer.DefaultPicoContainer;
import org.picocontainer.MutablePicoContainer;
import org.picocontainer.behaviors.OptInCaching;
import org.picocontainer.lifecycle.ReflectionLifecycleStrategy;
import org.picocontainer.monitors.NullComponentMonitor;
import org.sonar.commons.database.DatabaseConnector;
import org.sonar.commons.database.JndiDatabaseConnector;
import org.sonar.core.batch.monitoring.MonitorImpl;
import org.sonar.core.database.EmbeddedDatabaseFactory;
import org.sonar.plugins.api.PluginsClasspathIntrospector;
import org.sonar.web.maven.MavenRepositoryBuilder;

import javax.servlet.ServletContextEvent;

public class Application {
  private static final Application singleton = new Application();

  protected MutablePicoContainer pico;

  private Application() {

  }

  public static Application getInstance() {
    return singleton;
  }

  public void start(ServletContextEvent event) {
    // components are started/stopped by reflection. It executed the following methods :
    ReflectionLifecycleStrategy lifecycleStrategy = new ReflectionLifecycleStrategy(new
      NullComponentMonitor(), "start", "stop", "dispose");

    // one instance per component
    pico = new DefaultPicoContainer(new OptInCaching(), lifecycleStrategy, null);
    pico.as(Characteristics.CACHE).addComponent(new ConfigurationFactory().getConfiguration(event));
    pico.as(Characteristics.CACHE).addComponent(JRubyOnRailsPropertiesInjector.class);
    pico.as(Characteristics.CACHE).addComponent(EmbeddedDatabaseFactory.class);
    pico.as(Characteristics.CACHE).addComponent(JndiDatabaseConnector.class);
    pico.as(Characteristics.CACHE).addComponent(MonitorImpl.class);
    pico.as(Characteristics.CACHE).addComponent(MavenRepositoryBuilder.class);
    pico.as(Characteristics.CACHE).addComponent(PluginsClasspathIntrospector.class);
    pico.start();

    OperationalDatabaseListener databaseListener = new OperationalDatabaseListener(pico, pico.getComponent(DatabaseConnector.class));
    databaseListener.start();
    pico.as(Characteristics.CACHE).addComponent(databaseListener);
  }

  public void stop() {
    if (pico != null) {
      pico.stop();
    }
  }

  public MutablePicoContainer getContainer() {
    return pico;
  }
}
